==> pytactoe-0.4.1.tar.gz: pytactoe-0.4.1/pytactoe/tictactoe.py <==
from __future__ import annotations

from pytactoe.board import Piece, Board

class TicTacToe(Board):
    ''' Game logic of TicTacToe. '''
    def __init__(self, m=3, n=3, k=3, infinite=False) -> None:
        ''' Initialize an (m, n, k)-game. '''
        super().__init__(m, n, infinite)
        self.k = k
        self.turn = Piece(1)
        self.ended = False
        self.draw = False
        self.winned = False
        self.winner = Piece()
        self.move_count = 0
        self.move_history = []

    def copy(self, game: TicTacToe):
        self.__dict__ = game.__dict__.copy()

    def checkdraw(self) -> bool:
        ''' Return True and update attributes if draw. '''
        if self.winned == True:
            return False
        if Piece() not in self.flat():
            self.ended = True
            self.draw = True
            return True
        return False

    @staticmethod
    def check_line_winned(line: list, k=3) -> bool:
        ''' Return True if line contain k-in-a-row. '''
        for elem in line:
            if not isinstance(elem, Piece):
                raise ValueError('Line contains non-piece objects.')

        if k > len(line):
            return False

        for i in range(len(line)-k+1):
            next_k_fields = line[i:i+k]
            if Piece() in next_k_fields:
                continue
            if all(field==next_k_fields[0] for field in next_k_fields):
                return True

        return False

    def checkwin(self) -> bool:
        ''' Return True and update attributes if win. '''
        check = False
        for line in self.lines():
            check = TicTacToe.check_line_winned(line, self.k)
            if check:
                self.winner = self.turn
                self.winned = True
                self.ended = True
                break
        return check

    def coords_winnedline(self) -> list:
        ''' Return coordinates of a winned line. The line is chosen as the first in self.lines() list.  '''
        # NOT IMPLEMENTED
        # needed for gui to highlight winned line
        pass
        
    def changeturn(self) -> None:
        ''' Reverse turn attribute. '''
        self.turn = Piece.reverse(self.turn)

    def revertturn(self) -> None:
        ''' Revert turn attribute. '''
        self.turn = Piece.previous(self.turn)
    
    def movevalid(self, x, y) -> bool:
        ''' Return True if move is valid. '''
        if not self.ended:
            return self[x, y] == Piece()
        return False

    def move(self, x, y) -> bool:
        ''' Make move and update attributes. Return True if move is successful. '''
        if self.movevalid(x, y):
            self[x, y] = self.turn
            self.checkwin()
            self.checkdraw()
            self.changeturn()
            self.move_history.append((x,y))
            self.move_count += 1
            return True
        return False

    def pop(self) -> None:
        ''' Revert board to the previous position. '''
        if not self.move_count:
            return

        last_move = self.move_history[-1]

        self.draw = False
        self.ended = False
        self.winned = False
        self.winner = Piece()
        self.move_count -= 1
        self.move_history.pop()
        self[last_move[0], last_move[1]] = Piece()

        self.revertturn()
        self.checkwin()
        self.checkdraw()

    def legalmoves(self) -> list:
        ''' Return list of tuples of all legal move coordinates. '''
        return [(i,j) for (i,j) in self.moves_coords() if self.movevalid(i,j)]

    def __str__(self) -> str:
        return super().__str__() + f'(m,n,k): ({self.m},{self.n},{self.k}), infinite: {self.infinite}, '\
            f'turn: {self.turn}, ended: {self.ended}, winned: {self.winned}, winner: {self.winner}, draw: {self.draw}\n'

==> pytactoe-0.7.0.tar.gz: pytactoe-0.7.0/pytactoe/tictactoe.py <==
from __future__ import annotations

from pytactoe.board import Piece, Board

class TicTacToe(Board):
    ''' Game logic of TicTacToe. '''
    def __init__(self, m=3, n=3, k=3, infinite=False) -> None:
        ''' Initialize an (m, n, k)-game. '''
        super().__init__(m, n, infinite)
        self.k = k
        self.turn = Piece(1)
        self.ended = False
        self.draw = False
        self.winned = False
        self.winner = 0
        self.move_count = 0
        self.move_history = []

    def copy(self, game: TicTacToe):
        self.__dict__ = game.__dict__.copy()

    def checkdraw(self) -> bool:
        ''' Return True and update attributes if draw. '''
        if self.winned == True:
            return False
        if Piece() not in self.flat():
            self.ended = True
            self.draw = True
            return True
        return False

    @staticmethod
    def check_line_winned(line: list, k=3) -> bool:
        ''' Return True if line contain k-in-a-row. '''
        for elem in line:
            if not isinstance(elem, Piece):
                raise ValueError('Line contains non-piece objects.')

        if k > len(line):
            return False

        for i in range(len(line)-k+1):
            next_k_fields = line[i:i+k]
            if Piece() in next_k_fields:
                continue
            if all(field==next_k_fields[0] for field in next_k_fields):
                return True

        return False

    def checkwin(self) -> bool:
        ''' Return True and update attributes if win. '''
        winned = False
        for line in self.lines():
            winned = TicTacToe.check_line_winned(line, self.k)
            if winned:
                self.winner = self.move_count%(Piece.playernumber)+1
                self.winned = True
                self.ended = True
                break
        return winned

    def coords_winnedline(self) -> list:
        ''' Return coordinates of a winned line. The line is chosen as the first in self.lines() list.  '''
        # NOT IMPLEMENTED
        # needed for gui to highlight winned line
        pass
        
    def changeturn(self) -> None:
        ''' Reverse turn attribute. '''
        self.turn = Piece.reverse(self.turn)

    def revertturn(self) -> None:
        ''' Revert turn attribute. '''
        self.turn = Piece.previous(self.turn)
    
    def movevalid(self, x, y) -> bool:
        ''' Return True if move is valid. '''
        if not self.ended:
            return self[x, y] == Piece()
        return False

    def move(self, x, y) -> bool:
        ''' Make move and update attributes. Return True if move is successful. '''
        if self.movevalid(x, y):
            self[x, y] = self.turn
            self.checkwin()
            self.checkdraw()
            self.changeturn()
            self.move_history.append((x,y))
            self.move_count += 1
            return True
        return False

    def pop(self) -> None:
        ''' Revert board to the previous position. '''
        if not self.move_count:
            return

        last_move = self.move_history[-1]

        self.draw = False
        self.ended = False
        self.winned = False
        self.winner = Piece()
        self.move_count -= 1
        self.move_history.pop()
        self[last_move[0], last_move[1]] = Piece()

        self.revertturn()
        self.checkwin()
        self.checkdraw()

    def legalmoves(self) -> list:
        ''' Return list of tuples of all legal move coordinates. '''
        return [(i,j) for (i,j) in self.moves_coords() if self.movevalid(i,j)]

    def __str__(self) -> str:
        return super().__str__() + f'(m,n,k): ({self.m},{self.n},{self.k}), infinite: {self.infinite}, '\
            f'turn: {self.turn}, ended: {self.ended}, winned: {self.winned}, winner: {self.winner}, draw: {self.draw}\n'
